@@ -2,6 +2,7 @@
  * Copyright (C) 2005, Red Hat, Inc.
  *
  * Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
+ * Copyright (C) 2018 Marek Kasik <mkasik@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,6 +37,7 @@
 #include <FontInfo.h>
 #include <PDFDocEncoding.h>
 #include <OptionalContent.h>
+#include <ViewerPreferences.h>
 #endif
 
 #include "poppler.h"
@@ -78,7 +80,8 @@ enum {
 	PROP_PAGE_MODE,
 	PROP_VIEWER_PREFERENCES,
 	PROP_PERMISSIONS,
-	PROP_METADATA
+	PROP_METADATA,
+	PROP_PRINT_SCALING
 };
 
 static void poppler_document_layers_free (PopplerDocument *document);
@@ -680,41 +683,154 @@ poppler_document_get_attachments (PopplerDocument *document)
   return g_list_reverse (retval);
 }
 
+/**
+ * poppler_named_dest_from_bytestring:
+ * @data: (array length=length): the bytestring data
+ * @length: the bytestring length
+ *
+ * Converts a bytestring into a zero-terminated string suitable to
+ * pass to poppler_document_find_dest().
+ *
+ * Note that the returned string has no defined encoding and is not
+ * suitable for display to the user.
+ *
+ * The returned data must be freed using g_free().
+ *
+ * Returns: (transfer full): the named dest
+ *
+ * Since: 0.73
+ */
+char *
+poppler_named_dest_from_bytestring (const guint8 *data,
+				    gsize         length)
+{
+  const guint8 *p, *pend;
+  char *dest, *q;
+
+  g_return_val_if_fail (length != 0 || data != nullptr, nullptr);
+  /* Each source byte needs maximally 2 destination chars (\\ or \0) */
+  q = dest = (gchar *)g_malloc (length * 2 + 1);
+
+  pend = data + length;
+  for (p = data; p < pend; ++p) {
+    switch (*p) {
+    case '\0':
+      *q++ = '\\';
+      *q++ = '0';
+      break;
+    case '\\':
+      *q++ = '\\';
+      *q++ = '\\';
+      break;
+    default:
+      *q++ = *p;
+      break;
+    }
+  }
+
+  *q = 0; /* zero terminate */
+  return dest;
+}
+
+/**
+ * poppler_named_dest_to_bytestring:
+ * @name: the named dest string
+ * @length: (out): a location to store the length of the returned bytestring
+ *
+ * Converts a named dest string (e.g. from #PopplerDest.named_dest) into a
+ * bytestring, inverting the transformation of
+ * poppler_named_dest_from_bytestring().
+ *
+ * Note that the returned data is not zero terminated and may also
+ * contains embedded NUL bytes.
+ *
+ * If @name is not a valid named dest string, returns %NULL.
+ *
+ * The returned data must be freed using g_free().
+ *
+ * Returns: (array length=length) (transfer full) (nullable): a new bytestring,
+ *   or %NULL
+ *
+ * Since: 0.73
+ */
+guint8 *
+poppler_named_dest_to_bytestring (const char *name,
+				  gsize      *length)
+{
+  const char *p;
+  guint8 *data, *q;
+  gsize len;
+
+  g_return_val_if_fail (name != nullptr, nullptr);
+  g_return_val_if_fail (length != nullptr, nullptr);
+
+  len = strlen (name);
+  q = data = (guint8*) g_malloc (len);
+  for (p = name; *p; ++p) {
+    if (*p == '\\') {
+      p++;
+      len--;
+      if (*p == '0')
+	*q++ = '\0';
+      else if (*p == '\\')
+	*q++ = '\\';
+      else
+	goto invalid;
+    } else {
+      *q++ = *p;
+    }
+  }
+
+  *length = len;
+  return data;
+
+invalid:
+  g_free(data);
+  *length = 0;
+  return nullptr;
+}
+
 /**
  * poppler_document_find_dest:
  * @document: A #PopplerDocument
  * @link_name: a named destination
  *
- * Finds named destination @link_name in @document
+ * Creates a #PopplerDest for the named destination @link_name in @document.
  *
- * Return value: The #PopplerDest destination or %NULL if
- * @link_name is not a destination. Returned value must
- * be freed with #poppler_dest_free
+ * Note that named destinations are bytestrings, not string. That means that
+ * unless @link_name was returned by a poppler function (e.g. is
+ * #PopplerDest.named_dest), it needs to be converted to string
+ * using poppler_named_dest_from_bytestring() before being passed to this
+ * function.
+ *
+ * The returned value must be freed with poppler_dest_free().
+ *
+ * Return value: (transfer full): a new #PopplerDest destination, or %NULL if
+ *   @link_name is not a destination.
  **/
 PopplerDest *
 poppler_document_find_dest (PopplerDocument *document,
 			    const gchar     *link_name)
 {
-	PopplerDest *dest = nullptr;
-	LinkDest *link_dest = nullptr;
-	GooString *g_link_name;
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), nullptr);
+  g_return_val_if_fail (link_name != nullptr, nullptr);
 
-	g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), NULL);
-	g_return_val_if_fail (link_name != nullptr, NULL);
+  gsize len;
+  guint8* data = poppler_named_dest_to_bytestring (link_name, &len);
+  if (data == nullptr)
+    return nullptr;
 
-	g_link_name = new GooString (link_name);
+  GooString g_link_name ((const char*)data, (int)len);
+  g_free (data);
 
-	if (g_link_name) {
-		link_dest = document->doc->findDest (g_link_name);
-		delete g_link_name;
-	}
+  LinkDest *link_dest = document->doc->findDest (&g_link_name);
+  if (link_dest == nullptr)
+    return nullptr;
 
-	if (link_dest) {
-		dest = _poppler_dest_new_goto (document, link_dest);
-		delete link_dest;
-	}
+  PopplerDest *dest = _poppler_dest_new_goto (document, link_dest);
+  delete link_dest;
 
-	return dest;
+  return dest;
 }
 
 char *_poppler_goo_string_to_utf8(const GooString *s)
@@ -1342,7 +1458,7 @@ poppler_document_set_modification_date (PopplerDocument *document,
  * Returns whether @document is linearized or not. Linearization of PDF
  * enables efficient incremental access of the PDF file in a network environment.
  *
- * Return value: %TRUE if @document is linearized, %FALSE otherwhise
+ * Return value: %TRUE if @document is linearized, %FALSE otherwise
  *
  * Since: 0.16
  **/
@@ -1403,13 +1519,51 @@ poppler_document_get_page_mode (PopplerDocument *document)
   return POPPLER_PAGE_MODE_UNSET;
 }
 
+/**
+ * poppler_document_get_print_scaling:
+ * @document: A #PopplerDocument
+ *
+ * Returns the print scaling value suggested by author of the document.
+ *
+ * Return value: a #PopplerPrintScaling that should be used when document is printed
+ *
+ * Since: 0.73
+ **/
+PopplerPrintScaling
+poppler_document_get_print_scaling (PopplerDocument *document)
+{
+  Catalog *catalog;
+  ViewerPreferences *preferences;
+  PopplerPrintScaling print_scaling = POPPLER_PRINT_SCALING_APP_DEFAULT;
+
+  g_return_val_if_fail (POPPLER_IS_DOCUMENT (document), POPPLER_PRINT_SCALING_APP_DEFAULT);
+
+  catalog = document->doc->getCatalog ();
+  if (catalog && catalog->isOk ()) {
+    preferences = catalog->getViewerPreferences();
+    if (preferences) {
+      switch (preferences->getPrintScaling()) {
+        default:
+        case ViewerPreferences::PrintScaling::printScalingAppDefault:
+          print_scaling = POPPLER_PRINT_SCALING_APP_DEFAULT;
+          break;
+        case ViewerPreferences::PrintScaling::printScalingNone:
+          print_scaling = POPPLER_PRINT_SCALING_NONE;
+          break;
+      }
+    }
+  }
+
+  return print_scaling;
+}
+
 /**
  * poppler_document_get_permissions:
  * @document: A #PopplerDocument
  *
  * Returns the flags specifying which operations are permitted when the document is opened.
  *
- * Return value: a set of falgs from  #PopplerPermissions enumeration
+ * Return value: a set of flags from  #PopplerPermissions enumeration
  *
  * Since: 0.16
  **/
@@ -1633,6 +1787,9 @@ poppler_document_get_property (GObject    *object,
       /* FIXME: write... */
       g_value_set_flags (value, POPPLER_VIEWER_PREFERENCES_UNSET);
       break;
+    case PROP_PRINT_SCALING:
+      g_value_set_enum (value, poppler_document_get_print_scaling (document));
+      break;
     case PROP_PERMISSIONS:
       g_value_set_flags (value, poppler_document_get_permissions (document));
       break;
@@ -1900,6 +2057,20 @@ poppler_document_class_init (PopplerDocumentClass *klass)
 						       POPPLER_VIEWER_PREFERENCES_UNSET,
 						       G_PARAM_READABLE));
 
+  /**
+   * PopplerDocument:print-scaling:
+   *
+   * Since: 0.73
+   */
+  g_object_class_install_property (G_OBJECT_CLASS (klass),
+				   PROP_PRINT_SCALING,
+				   g_param_spec_enum ("print-scaling",
+						      "Print Scaling",
+						      "Print Scaling Viewer Preference",
+						      POPPLER_TYPE_PRINT_SCALING,
+						      POPPLER_PRINT_SCALING_APP_DEFAULT,
+						      (GParamFlags) (G_PARAM_READABLE | G_PARAM_STATIC_STRINGS)));
+
   /**
    * PopplerDocument:permissions:
    *
@@ -2044,7 +2215,7 @@ poppler_index_iter_copy (PopplerIndexIter *iter)
  * {
  *   do
  *     {
- *       /<!-- -->* Get the the action and do something with it *<!-- -->/
+ *       /<!-- -->* Get the action and do something with it *<!-- -->/
  *       PopplerIndexIter *child = poppler_index_iter_get_child (iter);
  *       if (child)
  *         walk_index (child);
@@ -2394,7 +2565,7 @@ poppler_fonts_iter_get_encoding (PopplerFontsIter *iter)
  *
  * Returns whether the font associated with @iter is embedded in the document
  *
- * Returns: %TRUE if font is emebdded, %FALSE otherwise
+ * Returns: %TRUE if font is embedded, %FALSE otherwise
  */
 gboolean
 poppler_fonts_iter_is_embedded (PopplerFontsIter *iter)
@@ -2674,7 +2845,7 @@ get_optional_content_rbgroups (OCGs *ocg)
       for (j = 0; j < rb_array->getLength (); ++j) {
 	OptionalContentGroup *oc;
 
-        Object ref = rb_array->getNF (j);
+        const Object &ref = rb_array->getNF (j);
 	if (!ref.isRef ()) {
 	  continue;
 	}
@@ -2717,7 +2888,7 @@ get_optional_content_items_sorted (OCGs *ocg, Layer *parent, Array *order)
     Object orderItem = order->get (i);
 
     if (orderItem.isDict ()) {
-      Object ref = order->getNF (i);
+      const Object &ref = order->getNF (i);
       if (ref.isRef ()) {
         OptionalContentGroup *oc = ocg->findOcgByRef (ref.getRef ());
 	Layer *layer = layer_new (oc);
