@@ -14,7 +14,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2005 Kristian Høgsberg <krh@redhat.com>
-// Copyright (C) 2005-2013, 2015, 2017, 2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2013, 2015, 2017-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005 Jeff Muizelaar <jrmuizel@nit.ca>
 // Copyright (C) 2005 Jonathan Blandford <jrb@redhat.com>
 // Copyright (C) 2005 Marco Pesenti Gritti <mpg@redhat.com>
@@ -120,7 +120,7 @@ Catalog::Catalog(PDFDoc *docA) {
   }
 
   // actions
-  additionalActions = catDict.dictLookupNF("AA");
+  additionalActions = catDict.dictLookupNF("AA").copy();
 
   // get the ViewerPreferences dictionary
   viewerPreferences = catDict.dictLookup("ViewerPreferences");
@@ -213,7 +213,7 @@ bool Catalog::cachePageTree(int page)
     Object catDict = xref->getCatalog();
 
     if (catDict.isDict()) {
-      Object pagesDictRef = catDict.dictLookupNF("Pages");
+      const Object &pagesDictRef = catDict.dictLookupNF("Pages");
       if (pagesDictRef.isRef() &&
           pagesDictRef.getRefNum() >= 0 &&
           pagesDictRef.getRefNum() < xref->getNumObjects()) {
@@ -270,7 +270,7 @@ bool Catalog::cachePageTree(int page)
        continue;
     }
 
-    Object kidRef = kids.arrayGetNF(kidsIdx);
+    const Object &kidRef = kids.arrayGetNF(kidsIdx);
     if (!kidRef.isRef()) {
       error(errSyntaxError, -1, "Kid object (page {0:uld}) is not an indirect reference ({1:s})",
 	    pages.size()+1, kidRef.getTypeName());
@@ -432,6 +432,87 @@ FileSpec *Catalog::embeddedFile(int i)
     return embeddedFile;
 }
 
+bool Catalog::hasEmbeddedFile(const std::string &fileName)
+{
+  NameTree *ef = getEmbeddedFileNameTree();
+  for (int i = 0; i < ef->numEntries(); ++i) {
+    if (fileName == ef->getName(i)->toStr())
+      return true;
+  }
+  return false;
+}
+
+void Catalog::addEmbeddedFile(GooFile *file, const std::string &fileName)
+{
+  catalogLocker();
+
+  Object fileSpecObj = FileSpec::newFileSpecObject(xref, file, fileName);
+  const Ref fileSpecRef = xref->addIndirectObject(&fileSpecObj);
+
+  Object catDict = xref->getCatalog();
+  Ref namesObjRef;
+  Object namesObj = catDict.getDict()->lookup("Names", &namesObjRef);
+  if (!namesObj.isDict()) {
+    // Need to create the names Dict
+    catDict.dictSet("Names", Object(new Dict(xref)));
+    namesObj = catDict.getDict()->lookup("Names");
+
+    // Trigger getting the names dict again when needed
+    names = Object();
+  }
+
+  Dict *namesDict = namesObj.getDict();
+
+  // We create a new EmbeddedFiles nametree, this replaces the existing one (if any), but it's not a problem
+  Object embeddedFilesObj = Object(new Dict(xref));
+  const Ref embeddedFilesRef = xref->addIndirectObject(&embeddedFilesObj);
+
+  Array *embeddedFilesNamesArray = new Array(xref);
+
+  // This flattens out the existing EmbeddedFiles nametree (if any), should not be a problem
+  NameTree *ef = getEmbeddedFileNameTree();
+  bool fileAlreadyAdded = false;
+  for (int i = 0; i < ef->numEntries(); ++i) {
+    GooString *efNameI = ef->getName(i);
+
+    // we need to add the file if it has not been added yet and the name is smaller or equal lexicographically
+    // than the current item
+    const bool sameFileName = fileName == efNameI->toStr();
+    const bool addFile = !fileAlreadyAdded && (sameFileName || fileName < efNameI->toStr());
+    if (addFile) {
+      // If the new name is smaller lexicographically than an existing file add it in its correct position
+      embeddedFilesNamesArray->add(Object(new GooString(fileName)));
+      embeddedFilesNamesArray->add(Object(fileSpecRef));
+      fileAlreadyAdded = true;
+    }
+    if (sameFileName) {
+      // If the new name is the same lexicographically than an existing file then don't add the existing file (i.e. replace)
+      continue;
+    }
+    embeddedFilesNamesArray->add(Object(efNameI->copy()));
+    embeddedFilesNamesArray->add(ef->getValue(i)->copy());
+  }
+
+  if (!fileAlreadyAdded) {
+    // The new file is bigger lexicographically than the existing ones
+    embeddedFilesNamesArray->add(Object(new GooString(fileName)));
+    embeddedFilesNamesArray->add(Object(fileSpecRef));
+  }
+
+  embeddedFilesObj.dictSet("Names", Object(embeddedFilesNamesArray));
+  namesDict->set("EmbeddedFiles", Object(embeddedFilesRef));
+
+  if (namesObjRef.num != 0) {
+    xref->setModifiedObject(&namesObj, namesObjRef);
+  } else {
+    xref->setModifiedObject(&catDict, { xref->getRootNum(), xref->getRootGen() });
+  }
+
+  // recreate Nametree on next call that uses it
+  delete embeddedFileNameTree;
+  embeddedFileNameTree = nullptr;
+}
+
 GooString *Catalog::getJS(int i)
 {
   Object obj;
@@ -558,7 +639,7 @@ NameTree::Entry::Entry(Array *array, int index) {
     else
       error(errSyntaxError, -1, "Invalid page tree");
   }
-  value = array->getNF(index + 1);
+  value = array->getNF(index + 1).copy();
 }
 
 NameTree::Entry::~Entry() {
@@ -615,7 +696,7 @@ void NameTree::parse(Object *tree, std::set<int> &seen) {
   Object kids = tree->dictLookup("Kids");
   if (kids.isArray()) {
     for (int i = 0; i < kids.arrayGetLength(); ++i) {
-      Object kidRef = kids.arrayGetNF(i);
+      const Object &kidRef = kids.arrayGetNF(i);
       if (kidRef.isRef()) {
 	const int numObj = kidRef.getRef().num;
 	if (seen.find(numObj) != seen.end()) {
@@ -732,7 +813,7 @@ int Catalog::getNumPages()
     // some PDF files actually use real numbers here ("/Count 9.0")
     if (!obj.isNum()) {
       if (pagesDict.dictIs("Page")) {
-	Object pageRootRef = catDict.dictLookupNF("Pages");
+	const Object &pageRootRef = catDict.dictLookupNF("Pages");
 
 	error(errSyntaxError, -1, "Pages top-level is a single Page. The document is malformed, trying to recover...");
 
@@ -811,7 +892,7 @@ StructTreeRoot *Catalog::getStructTreeRoot()
   return structTreeRoot;
 }
 
-Guint Catalog::getMarkInfo()
+unsigned int Catalog::getMarkInfo()
 {
   if (markInfo == markInfoNull) {
     markInfo = 0;
