@@ -99,7 +99,7 @@ static int hexCharVals[256] = {
 
 // Parse a <len>-byte hex string <s> into *<val>.  Returns false on
 // error.
-static bool parseHex(const char *s, int len, Guint *val) {
+static bool parseHex(const char *s, int len, unsigned int *val) {
   int i, x;
 
   *val = 0;
@@ -622,7 +622,7 @@ int CharCodeToUnicode::mapToCharCode(Unicode* u, CharCode *c, int usize) const {
     //for each entry in the sMap
     for (i=0; i<sMapLen; i++) {
       //if the entry's unicode length isn't the same are usize, the strings 
-      // are obviously differents
+      // are obviously different
       if (sMap[i].len != usize) continue;
       //compare the string char by char
       for (j=0; j<sMap[i].len; j++) {
