@@ -3,7 +3,7 @@
 // FontInfo.cc
 //
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
-// Copyright (C) 2005-2008, 2010, 2017, 2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2008, 2010, 2017-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
 // Copyright (C) 2006 Kouhei Sutou <kou@cozmixng.org>
 // Copyright (C) 2009 Pino Toscano <pino@kde.org>
@@ -99,15 +99,15 @@ void FontInfoScanner::scanFonts(XRef *xrefA, Dict *resDict, GooList *fontsList)
 
   // scan the fonts in this resource dictionary
   gfxFontDict = nullptr;
-  Object obj1 = resDict->lookupNF("Font");
-  if (obj1.isRef()) {
-    Object obj2 = obj1.fetch(xrefA);
+  const Object &fontObj = resDict->lookupNF("Font");
+  if (fontObj.isRef()) {
+    Object obj2 = fontObj.fetch(xrefA);
     if (obj2.isDict()) {
-      Ref r = obj1.getRef();
+      Ref r = fontObj.getRef();
       gfxFontDict = new GfxFontDict(xrefA, &r, obj2.getDict());
     }
-  } else if (obj1.isDict()) {
-    gfxFontDict = new GfxFontDict(xrefA, nullptr, obj1.getDict());
+  } else if (fontObj.isDict()) {
+    gfxFontDict = new GfxFontDict(xrefA, nullptr, fontObj.getDict());
   }
   if (gfxFontDict) {
     for (int i = 0; i < gfxFontDict->getNumFonts(); ++i) {
@@ -131,10 +131,10 @@ void FontInfoScanner::scanFonts(XRef *xrefA, Dict *resDict, GooList *fontsList)
     Object objDict = resDict->lookup(resTypes[resType]);
     if (objDict.isDict()) {
       for (int i = 0; i < objDict.dictGetLength(); ++i) {
-        obj1 = objDict.dictGetValNF(i);
-        if (obj1.isRef()) {
+        const Object &resObj = objDict.dictGetValNF(i);
+        if (resObj.isRef()) {
           // check for an already-seen object
-          const Ref r = obj1.getRef();
+          const Ref r = resObj.getRef();
           if (visitedObjects.find(r.num) != visitedObjects.end()) {
             continue;
           }
@@ -142,7 +142,7 @@ void FontInfoScanner::scanFonts(XRef *xrefA, Dict *resDict, GooList *fontsList)
           visitedObjects.insert(r.num);
         }
 
-        Object obj2 = obj1.fetch(xrefA);
+        Object obj2 = resObj.fetch(xrefA);
         if (obj2.isStream()) {
           Object resObj = obj2.streamGetDict()->lookup("Resources");
           if (resObj.isDict() && resObj.getDict() != resDict) {
@@ -192,7 +192,7 @@ FontInfo::FontInfo(GfxFont *font, XRef *xref) {
 
   // look for a ToUnicode map
   hasToUnicode = false;
-  Object fontObj = xref->fetch(fontRef.num, fontRef.gen);
+  Object fontObj = xref->fetch(fontRef);
   if (fontObj.isDict()) {
     hasToUnicode = fontObj.dictLookup("ToUnicode").isStream();
   }
