@@ -13,7 +13,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2005, 2006, 2008-2010, 2012, 2014, 2015, 2017, 2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005, 2006, 2008-2010, 2012, 2014, 2015, 2017-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2005, 2006 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2006 Takashi Iwai <tiwai@suse.de>
 // Copyright (C) 2007 Julien Rebetez <julienr@svn.gnome.org>
@@ -34,6 +34,7 @@
 // Copyright (C) 2014 Olly Betts <olly@survex.com>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
+// Copyright (C) 2019 LE GARREC Vincent <legarrec.vincent@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -340,14 +341,14 @@ GfxFontType GfxFont::getFontType(XRef *xref, Dict *fontDict, Ref *embID) {
 
   Object fontDesc = fontDict2->lookup("FontDescriptor");
   if (fontDesc.isDict()) {
-    Object obj3 = fontDesc.dictLookupNF("FontFile");
+    Object obj3 = fontDesc.dictLookupNF("FontFile").copy();
     if (obj3.isRef()) {
       *embID = obj3.getRef();
       if (expectedType != fontType1) {
 	err = true;
       }
     }
-    if (embID->num == -1 && (obj3 = fontDesc.dictLookupNF("FontFile2"), obj3.isRef())) {
+    if (embID->num == -1 && (obj3 = fontDesc.dictLookupNF("FontFile2").copy(), obj3.isRef())) {
       *embID = obj3.getRef();
       if (isType0) {
 	expectedType = fontCIDType2;
@@ -355,7 +356,7 @@ GfxFontType GfxFont::getFontType(XRef *xref, Dict *fontDict, Ref *embID) {
 	err = true;
       }
     }
-    if (embID->num == -1 && (obj3 = fontDesc.dictLookupNF("FontFile3"), obj3.isRef())) {
+    if (embID->num == -1 && (obj3 = fontDesc.dictLookupNF("FontFile3").copy(), obj3.isRef())) {
       *embID = obj3.getRef();
       Object obj4 = obj3.fetch(xref);
       if (obj4.isStream()) {
@@ -406,7 +407,7 @@ GfxFontType GfxFont::getFontType(XRef *xref, Dict *fontDict, Ref *embID) {
 
   t = fontUnknownType;
   if (embID->num >= 0) {
-    Object obj3(embID->num, embID->gen);
+    Object obj3(*embID);
     Object obj4 = obj3.fetch(xref);
     if (obj4.isStream()) {
       obj4.streamReset();
@@ -602,7 +603,7 @@ GfxFontLoc *GfxFont::locateFont(XRef *xref, PSOutputDev *ps) {
   //----- embedded font
   if (embFontID.num >= 0) {
     embed = true;
-    Object refObj(embFontID.num, embFontID.gen);
+    Object refObj(embFontID);
     Object embFontObj = refObj.fetch(xref);
     if (!embFontObj.isStream()) {
       error(errSyntaxError, -1, "Embedded font object is wrong type");
@@ -818,7 +819,7 @@ char *GfxFont::readEmbFontFile(XRef *xref, int *len) {
   char *buf;
   Stream *str;
 
-  Object obj1(embFontID.num, embFontID.gen);
+  Object obj1(embFontID);
   Object obj2 = obj1.fetch(xref);
   if (!obj2.isStream()) {
     error(errSyntaxError, -1, "Embedded font file is not a stream");
@@ -930,7 +931,7 @@ static bool testForNumericNames(Dict *fontDict, bool hex) {
       if (obj.getInt() > 5)
 	numeric = false;
     } else if (obj.isName()) {
-      // All character names must sucessfully parse.
+      // All character names must successfully parse.
       if (!parseNumericName(obj.getName(), hex, nullptr))
 	numeric = false;
     } else {
@@ -952,7 +953,6 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
   int len;
   FoFiType1 *ffT1;
   FoFiType1C *ffT1C;
-  int code;
   char *charName;
   bool missing, hex;
   bool numeric;
@@ -960,7 +960,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
   Unicode uBuf[8];
   double mul;
   int firstChar, lastChar;
-  Gushort w;
+  unsigned short w;
   Object obj1;
   int n, i, a, b, m;
 
@@ -1212,7 +1212,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
     if (obj2.isArray()) {
       encodingName->Set("Custom");
       hasEncoding = true;
-      code = 0;
+      int code = 0;
       for (i = 0; i < obj2.arrayGetLength(); ++i) {
 	Object obj3 = obj2.arrayGet(i);
 	if (obj3.isInt()) {
@@ -1224,8 +1224,8 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
 	    }
 	    enc[code] = copyString(obj3.getName());
 	    encFree[code] = true;
+	    ++code;
 	  }
-	  ++code;
 	} else {
 	  error(errSyntaxError, -1,
 		"Wrong type in font encoding resource differences ({0:s})",
@@ -1242,7 +1242,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
   // pass 1: use the name-to-Unicode mapping table
   missing = hex = false;
   bool isZapfDingbats = name && name->endsWith("ZapfDingbats");
-  for (code = 0; code < 256; ++code) {
+  for (int code = 0; code < 256; ++code) {
     if ((charName = enc[code])) {
       if (isZapfDingbats) {
 	// include ZapfDingbats names
@@ -1283,7 +1283,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
 
   // pass 1a: Expand ligatures in the Alphabetic Presentation Form
   // block (eg "fi", "ffi") to normal form
-  for (code = 0; code < 256; ++code) {
+  for (int code = 0; code < 256; ++code) {
     if (unicodeIsAlphabeticPresentationForm(toUnicode[code])) {
       Unicode *normalized = unicodeNormalizeNFKC(&toUnicode[code], 1, &len, nullptr);
       if (len > 1)
@@ -1295,7 +1295,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
   // pass 2: try to fill in the missing chars, looking for ligatures, numeric
   // references and variants
   if (missing) {
-    for (code = 0; code < 256; ++code) {
+    for (int code = 0; code < 256; ++code) {
       if (!toUnicode[code]) {
 	if ((charName = enc[code]) && strcmp(charName, ".notdef")
 	    && (n = parseCharName(charName, uBuf, sizeof(uBuf)/sizeof(*uBuf), 
@@ -1325,7 +1325,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
   //----- get the character widths -----
 
   // initialize all widths
-  for (code = 0; code < 256; ++code) {
+  for (int code = 0; code < 256; ++code) {
     widths[code] = missingWidth * 0.001;
   }
 
@@ -1347,7 +1347,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
     if (obj1.arrayGetLength() < lastChar - firstChar + 1) {
       lastChar = firstChar + obj1.arrayGetLength() - 1;
     }
-    for (code = firstChar; code <= lastChar; ++code) {
+    for (int code = firstChar; code <= lastChar; ++code) {
       Object obj2 = obj1.arrayGet(code - firstChar);
       if (obj2.isNum()) {
 	widths[code] = obj2.getNum() * mul;
@@ -1364,7 +1364,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
     if (builtinFont->widths->getWidth("space", &w)) {
       widths[32] = 0.001 * w;
     }
-    for (code = 0; code < 256; ++code) {
+    for (int code = 0; code < 256; ++code) {
       if (enc[code] && builtinFont->widths->getWidth(enc[code], &w)) {
 	widths[code] = 0.001 * w;
       }
@@ -1394,7 +1394,7 @@ Gfx8BitFont::Gfx8BitFont(XRef *xref, const char *tagA, Ref idA, GooString *nameA
     if (builtinFont->widths->getWidth("space", &w)) {
       widths[32] = 0.001 * w;
     }
-    for (code = 0; code < 256; ++code) {
+    for (int code = 0; code < 256; ++code) {
       if (enc[code] && builtinFont->widths->getWidth(enc[code], &w)) {
 	widths[code] = 0.001 * w;
       }
@@ -1684,7 +1684,7 @@ Object Gfx8BitFont::getCharProc(int code) {
 
 Object Gfx8BitFont::getCharProcNF(int code) {
   if (enc[code] && charProcs.isDict()) {
-    return charProcs.dictLookupNF(enc[code]);
+    return charProcs.dictLookupNF(enc[code]).copy();
   } else {
     return Object(objNull);
   }
@@ -2089,9 +2089,9 @@ const GooString *GfxCIDFont::getCollection() const {
 
 int GfxCIDFont::mapCodeToGID(FoFiTrueType *ff, int cmapi,
   Unicode unicode, bool wmode) {
-  Gushort gid = ff->mapCodeToGID(cmapi,unicode);
+  unsigned short gid = ff->mapCodeToGID(cmapi,unicode);
   if (wmode) {
-    Gushort vgid = ff->mapToVertGID(gid);
+    unsigned short vgid = ff->mapToVertGID(gid);
     if (vgid != 0) gid = vgid;
   }
   return gid;
@@ -2388,7 +2388,7 @@ GfxFontDict::GfxFontDict(XRef *xref, Ref *fontDictRef, Dict *fontDict) {
   numFonts = fontDict->getLength();
   fonts = (GfxFont **)gmallocn(numFonts, sizeof(GfxFont *));
   for (i = 0; i < numFonts; ++i) {
-    Object obj1 = fontDict->getValNF(i);
+    const Object &obj1 = fontDict->getValNF(i);
     Object obj2 = obj1.fetch(xref);
     if (obj2.isDict()) {
       if (obj1.isRef()) {
@@ -2469,7 +2469,7 @@ public:
 
 private:
 
-  Guint h;
+  unsigned int h;
 };
 
 int GfxFontDict::hashFontObject(Object *obj) {
@@ -2479,8 +2479,7 @@ int GfxFontDict::hashFontObject(Object *obj) {
   return h.get31();
 }
 
-void GfxFontDict::hashFontObject1(Object *obj, FNVHash *h) {
-  Object obj2;
+void GfxFontDict::hashFontObject1(const Object *obj, FNVHash *h) {
   const GooString *s;
   const char *p;
   double r;
@@ -2519,7 +2518,7 @@ void GfxFontDict::hashFontObject1(Object *obj, FNVHash *h) {
     n = obj->arrayGetLength();
     h->hash((char *)&n, sizeof(int));
     for (i = 0; i < n; ++i) {
-      obj2 = obj->arrayGetNF(i);
+      const Object &obj2 = obj->arrayGetNF(i);
       hashFontObject1(&obj2, h);
     }
     break;
@@ -2530,7 +2529,7 @@ void GfxFontDict::hashFontObject1(Object *obj, FNVHash *h) {
     for (i = 0; i < n; ++i) {
       p = obj->dictGetKey(i);
       h->hash(p, (int)strlen(p));
-      obj2 = obj->dictGetValNF(i);
+      const Object &obj2 = obj->dictGetValNF(i);
       hashFontObject1(&obj2, h);
     }
     break;
