@@ -13,7 +13,7 @@
 // All changes made under the Poppler project to this file are licensed
 // under GPL version 2 or later
 //
-// Copyright (C) 2008, 2010, 2012, 2017 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008, 2010, 2012, 2017, 2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2013 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 //
@@ -89,7 +89,7 @@ Object Object::fetch(XRef *xref, int recursion) const {
   CHECK_NOT_DEAD;
 
   return (type == objRef && xref) ?
-         xref->fetch(ref.num, ref.gen, recursion) : copy();
+         xref->fetch(ref, recursion) : copy();
 }
 
 void Object::free() {
@@ -127,7 +127,6 @@ const char *Object::getTypeName() const {
 }
 
 void Object::print(FILE *f) const {
-  Object obj;
   int i;
 
   switch (type) {
@@ -142,7 +141,7 @@ void Object::print(FILE *f) const {
     break;
   case objString:
     fprintf(f, "(");
-    fwrite(string->getCString(), 1, string->getLength(), f);
+    fwrite(string->c_str(), 1, string->getLength(), f);
     fprintf(f, ")");
     break;
   case objName:
@@ -156,7 +155,7 @@ void Object::print(FILE *f) const {
     for (i = 0; i < arrayGetLength(); ++i) {
       if (i > 0)
 	fprintf(f, " ");
-      obj = arrayGetNF(i);
+      const Object &obj = arrayGetNF(i);
       obj.print(f);
     }
     fprintf(f, "]");
@@ -165,7 +164,7 @@ void Object::print(FILE *f) const {
     fprintf(f, "<<");
     for (i = 0; i < dictGetLength(); ++i) {
       fprintf(f, " /%s ", dictGetKey(i));
-      obj = dictGetValNF(i);
+      const Object &obj = dictGetValNF(i);
       obj.print(f);
     }
     fprintf(f, " >>");
