@@ -15,7 +15,7 @@
 //
 // Copyright (C) 2005 Kristian Høgsberg <krh@redhat.com>
 // Copyright (C) 2005 Jeff Muizelaar <jeff@infidigm.net>
-// Copyright (C) 2005-2013, 2016-2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2013, 2016-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2006-2008 Pino Toscano <pino@kde.org>
 // Copyright (C) 2006 Nickolay V. Shmyrev <nshmyrev@yandex.ru>
 // Copyright (C) 2006 Scott Turner <scotty1024@mac.com>
@@ -258,7 +258,7 @@ Page::Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, PageAttrs *a
   attrs->clipBoxes();
 
   // transtion
-  trans = pageObj.dictLookupNF("Trans");
+  trans = pageObj.dictLookupNF("Trans").copy();
   if (!(trans.isRef() || trans.isDict() || trans.isNull())) {
     error(errSyntaxError, -1, "Page transition object (page {0:d}) is wrong type ({1:s})",
 	  num, trans.getTypeName());
@@ -266,7 +266,7 @@ Page::Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, PageAttrs *a
   }
 
   // duration
-  Object tmp = pageObj.dictLookupNF("Dur");
+  const Object &tmp = pageObj.dictLookupNF("Dur");
   if (!(tmp.isNum() || tmp.isNull())) {
     error(errSyntaxError, -1, "Page duration object (page {0:d}) is wrong type ({1:s})",
 	  num, tmp.getTypeName());
@@ -275,7 +275,7 @@ Page::Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, PageAttrs *a
   }
 
   // annotations
-  annotsObj = pageObj.dictLookupNF("Annots");
+  annotsObj = pageObj.dictLookupNF("Annots").copy();
   if (!(annotsObj.isRef() || annotsObj.isArray() || annotsObj.isNull())) {
     error(errSyntaxError, -1, "Page annotations object (page {0:d}) is wrong type ({1:s})",
 	  num, annotsObj.getTypeName());
@@ -283,7 +283,7 @@ Page::Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, PageAttrs *a
   }
 
   // contents
-  contents = pageObj.dictLookupNF("Contents");
+  contents = pageObj.dictLookupNF("Contents").copy();
   if (!(contents.isRef() || contents.isArray() ||
 	contents.isNull())) {
     error(errSyntaxError, -1, "Page contents object (page {0:d}) is wrong type ({1:s})",
@@ -292,7 +292,7 @@ Page::Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, PageAttrs *a
   }
 
   // thumb
-  thumb = pageObj.dictLookupNF("Thumb");
+  thumb = pageObj.dictLookupNF("Thumb").copy();
   if (!(thumb.isStream() || thumb.isNull() || thumb.isRef())) {
       error(errSyntaxError, -1, "Page thumb object (page {0:d}) is wrong type ({1:s})",
             num, thumb.getTypeName());
@@ -300,7 +300,7 @@ Page::Page(PDFDoc *docA, int numA, Object &&pageDict, Ref pageRefA, PageAttrs *a
   }
 
   // actions
-  actions = pageObj.dictLookupNF("AA");
+  actions = pageObj.dictLookupNF("AA").copy();
   if (!(actions.isDict() || actions.isNull())) {
       error(errSyntaxError, -1, "Page additional action object (page {0:d}) is wrong type ({1:s})",
             num, actions.getTypeName());
@@ -340,15 +340,15 @@ void Page::replaceXRef(XRef *xrefA) {
   Object obj1;
   Dict *pageDict = pageObj.getDict()->copy(xrefA);
   xref = xrefA;
-  trans = pageDict->lookupNF("Trans");
-  annotsObj = pageDict->lookupNF("Annots");
-  contents = pageDict->lookupNF("Contents");
+  trans = pageDict->lookupNF("Trans").copy();
+  annotsObj = pageDict->lookupNF("Annots").copy();
+  contents = pageDict->lookupNF("Contents").copy();
   if (contents.isArray()) {
-    obj1 = pageDict->lookupNF("Contents");
+    obj1 = pageDict->lookupNF("Contents").copy();
     contents = obj1.getArray()->copy(xrefA);
   }
-  thumb = pageDict->lookupNF("Thumb");
-  actions = pageDict->lookupNF("AA");
+  thumb = pageDict->lookupNF("Thumb").copy();
+  actions = pageDict->lookupNF("AA").copy();
   obj1 = pageDict->lookup("Resources");
   if (obj1.isDict()) {
     attrs->replaceResource(std::move(obj1));
@@ -366,7 +366,7 @@ Annots *Page::getAnnots(XRef *xrefA) {
 }
 
 void Page::addAnnot(Annot *annot) {
-  Ref annotRef = annot->getRef ();
+  const Ref annotRef = annot->getRef ();
 
   // Make sure we have annots before adding the new one
   // even if it's an empty list so that we can safely
@@ -380,16 +380,16 @@ void Page::addAnnot(Annot *annot) {
     // we have to create it
 
     Object obj1 = Object(new Array(xref));
-    obj1.arrayAdd(Object(annotRef.num, annotRef.gen));
+    obj1.arrayAdd(Object(annotRef));
 
     annotsRef = xref->addIndirectObject (&obj1);
-    annotsObj = Object(annotsRef.num, annotsRef.gen);
-    pageObj.dictSet ("Annots", Object(annotsRef.num, annotsRef.gen));
+    annotsObj = Object(annotsRef);
+    pageObj.dictSet ("Annots", Object(annotsRef));
     xref->setModifiedObject (&pageObj, pageRef);
   } else {
     Object obj1 = getAnnotsObject();
     if (obj1.isArray()) {
-      obj1.arrayAdd (Object(annotRef.num, annotRef.gen));
+      obj1.arrayAdd (Object(annotRef));
       if (annotsObj.isRef())
         xref->setModifiedObject (&obj1, annotsObj.getRef());
       else
@@ -423,7 +423,7 @@ void Page::removeAnnot(Annot *annot) {
     int idx = -1;
     // Get annotation position
     for (int i = 0; idx == -1 && i < annArray.arrayGetLength(); ++i) {
-      Object tmp = annArray.arrayGetNF(i);
+      const Object &tmp = annArray.arrayGetNF(i);
       if (tmp.isRef()) {
         Ref currAnnot = tmp.getRef();
         if (currAnnot.num == annotRef.num && currAnnot.gen == annotRef.gen) {
@@ -655,7 +655,7 @@ bool Page::loadThumb(unsigned char **data_out,
     imgstr->reset();
     for (int row = 0; row < height; ++row) {
       for (int col = 0; col < width; ++col) {
-        Guchar pix[gfxColorMaxComps];
+        unsigned char pix[gfxColorMaxComps];
         GfxRGB rgb;
 
         imgstr->getPixel(pix);
