@@ -15,7 +15,7 @@
 //
 // Copyright (C) 2005 Dan Sheridan <dan.sheridan@postman.org.uk>
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-// Copyright (C) 2006, 2008, 2010, 2012-2014, 2016-2018 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2008, 2010, 2012-2014, 2016-2019 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2007-2008 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2007 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009, 2010 Ilya Gorenbein <igorenbein@finjan.com>
@@ -25,7 +25,7 @@
 // Copyright (C) 2013, 2014, 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2013 Pino Toscano <pino@kde.org>
 // Copyright (C) 2016 Jakub Alba <jakubalba@gmail.com>
-// Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
+// Copyright (C) 2018, 2019 Adam Reichold <adam.reichold@t-online.de>
 // Copyright (C) 2018 Tobias Deiminger <haxtibal@posteo.de>
 //
 // To see a description of the changes please see the Changelog file that
@@ -291,7 +291,7 @@ XRef::XRef(BaseStream *strA, Goffset pos, Goffset mainXRefEntriesOffsetA, bool *
     }
 
     // set size to (at least) the size specified in trailer dict
-    obj = trailerDict.dictLookupNF("Size");
+    obj = trailerDict.dictLookupNF("Size").copy();
     if (!obj.isInt()) {
         error(errSyntaxWarning, -1, "No valid XRef size in trailer");
     } else {
@@ -306,7 +306,7 @@ XRef::XRef(BaseStream *strA, Goffset pos, Goffset mainXRefEntriesOffsetA, bool *
     }
 
     // get the root dictionary (catalog) object
-    obj = trailerDict.dictLookupNF("Root");
+    obj = trailerDict.dictLookupNF("Root").copy();
     if (obj.isRef()) {
       rootNum = obj.getRefNum();
       rootGen = obj.getRefGen();
@@ -588,7 +588,7 @@ bool XRef::readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *fol
   }
 
   // get the 'Prev' pointer
-  obj2 = obj.getDict()->lookupNF("Prev");
+  obj2 = obj.getDict()->lookupNF("Prev").copy();
   if (obj2.isInt() || obj2.isInt64()) {
     if (obj2.isInt())
       pos2 = obj2.getInt();
@@ -604,7 +604,7 @@ bool XRef::readXRefTable(Parser *parser, Goffset *pos, std::vector<Goffset> *fol
   } else if (obj2.isRef()) {
     // certain buggy PDF generators generate "/Prev NNN 0 R" instead
     // of "/Prev NNN"
-    pos2 = (Guint)obj2.getRefNum();
+    pos2 = (unsigned int)obj2.getRefNum();
     if (pos2 != *pos) {
       *pos = pos2;
       more = true;
@@ -657,7 +657,7 @@ bool XRef::readXRefStream(Stream *xrefStr, Goffset *pos) {
   ok = false;
 
   Dict *dict = xrefStr->getDict();
-  obj = dict->lookupNF("Size");
+  obj = dict->lookupNF("Size").copy();
   if (!obj.isInt()) {
     return false;
   }
@@ -672,7 +672,7 @@ bool XRef::readXRefStream(Stream *xrefStr, Goffset *pos) {
     }
   }
 
-  obj = dict->lookupNF("W");
+  obj = dict->lookupNF("W").copy();
   if (!obj.isArray() || obj.arrayGetLength() < 3) {
     return false;
   }
@@ -691,7 +691,7 @@ bool XRef::readXRefStream(Stream *xrefStr, Goffset *pos) {
   }
 
   xrefStr->reset();
-  Object idx = dict->lookupNF("Index");
+  const Object &idx = dict->lookupNF("Index");
   if (idx.isArray()) {
     for (int i = 0; i+1 < idx.arrayGetLength(); i += 2) {
       obj = idx.arrayGet(i);
@@ -715,7 +715,7 @@ bool XRef::readXRefStream(Stream *xrefStr, Goffset *pos) {
     }
   }
 
-  obj = dict->lookupNF("Prev");
+  obj = dict->lookupNF("Prev").copy();
   if (obj.isInt() && obj.getInt() >= 0) {
     *pos = obj.getInt();
     more = true;
@@ -816,7 +816,6 @@ bool XRef::readXRefStreamSection(Stream *xrefStr, int *w, int first, int n) {
 //          Existing data in XRef::entries may get corrupted if applied anyway.
 bool XRef::constructXRef(bool *wasReconstructed, bool needCatalogDict) {
   Parser *parser;
-  Object obj;
   char buf[256];
   Goffset pos;
   int num, gen;
@@ -872,7 +871,7 @@ bool XRef::constructXRef(bool *wasReconstructed, bool needCatalogDict) {
 		 false);
         Object newTrailerDict = parser->getObj();
         if (newTrailerDict.isDict()) {
-	  obj = newTrailerDict.dictLookupNF("Root");
+	  const Object &obj = newTrailerDict.dictLookupNF("Root");
 	  if (obj.isRef() && (!gotRoot || !needCatalogDict) && rootNum != obj.getRefNum()) {
 	    rootNum = obj.getRefNum();
 	    rootGen = obj.getRefGen();
@@ -979,7 +978,7 @@ bool XRef::constructXRef(bool *wasReconstructed, bool needCatalogDict) {
 }
 
 void XRef::setEncryption(int permFlagsA, bool ownerPasswordOkA,
-			 const Guchar *fileKeyA, int keyLengthA,
+			 const unsigned char *fileKeyA, int keyLengthA,
 			 int encVersionA, int encRevisionA,
 			 CryptAlgorithm encAlgorithmA) {
   int i;
@@ -1000,7 +999,7 @@ void XRef::setEncryption(int permFlagsA, bool ownerPasswordOkA,
   encAlgorithm = encAlgorithmA;
 }
 
-void XRef::getEncryptionParameters(Guchar **fileKeyA, CryptAlgorithm *encAlgorithmA,
+void XRef::getEncryptionParameters(unsigned char **fileKeyA, CryptAlgorithm *encAlgorithmA,
                               int *keyLengthA) {
   if (encrypted) {
     *fileKeyA = fileKey;
@@ -1072,6 +1071,11 @@ Object XRef::getCatalog() {
   return catalog;
 }
 
+Object XRef::fetch(const Ref ref, int recursion)
+{
+    return fetch(ref.num, ref.gen, recursion);
+}
+
 Object XRef::fetch(int num, int gen, int recursion) {
   XRefEntry *e;
   Parser *parser;
@@ -1141,7 +1145,7 @@ Object XRef::fetch(int num, int gen, int recursion) {
       goto err;
     }
 #endif
-    if (e->offset >= (Guint)size ||
+    if (e->offset >= (unsigned int)size ||
 	entries[e->offset].type != xrefEntryUncompressed) {
       error(errSyntaxError, -1, "Invalid object stream");
       goto err;
@@ -1191,7 +1195,7 @@ Object XRef::getDocInfo() {
 
 // Added for the pdftex project.
 Object XRef::getDocInfoNF() {
-  return trailerDict.dictLookupNF("Info");
+  return trailerDict.dictLookupNF("Info").copy();
 }
 
 Object XRef::createDocInfoIfNoneExists() {
@@ -1206,7 +1210,7 @@ Object XRef::createDocInfoIfNoneExists() {
 
   obj = Object(new Dict(this));
   const Ref ref = addIndirectObject(&obj);
-  trailerDict.dictSet("Info", Object(ref.num, ref.gen));
+  trailerDict.dictSet("Info", Object(ref));
 
   return obj;
 }
@@ -1548,11 +1552,36 @@ void XRef::readXRefUntil(int untilEntryNum, std::vector<int> *xrefStreamObjsNum)
   }
 }
 
+namespace {
+
+struct DummyXRefEntry : XRefEntry {
+  DummyXRefEntry() {
+    offset = 0;
+    gen = -1;
+    type = xrefEntryNone;
+    flags = 0;
+  }
+};
+
+DummyXRefEntry dummyXRefEntry;
+
+}
+
 XRefEntry *XRef::getEntry(int i, bool complainIfMissing)
 {
+  if (unlikely(i < 0)) {
+    error(errInternal, -1, "Request for invalid XRef entry [{0:d}]", i);
+    return &dummyXRefEntry;
+  }
+
   if (i >= size || entries[i].type == xrefEntryNone) {
 
     if ((!xRefStream) && mainXRefEntriesOffset) {
+      if (unlikely(i >= capacity)) {
+	error(errInternal, -1, "Request for out-of-bounds XRef entry [{0:d}]", i);
+	return &dummyXRefEntry;
+      }
+
       if (!parseEntry(mainXRefEntriesOffset + 20*i, &entries[i])) {
         error(errSyntaxError, -1, "Failed to parse XRef entry [{0:d}].", i);
       }
@@ -1563,12 +1592,7 @@ XRefEntry *XRef::getEntry(int i, bool complainIfMissing)
       // We might have reconstructed the xref
       // Check again i is in bounds
       if (unlikely(i >= size)) {
-        static XRefEntry dummy;
-        dummy.offset = 0;
-        dummy.gen = -1;
-        dummy.type = xrefEntryNone;
-        dummy.flags = 0;
-        return &dummy;
+	return &dummyXRefEntry;
       }
 
       if (entries[i].type == xrefEntryNone) {
@@ -1592,7 +1616,7 @@ void XRef::markUnencrypted(Object *obj) {
     {
       Array *array = obj->getArray();
       for (int i = 0; i < array->getLength(); i++) {
-	obj1 = array->getNF(i);
+	obj1 = array->getNF(i).copy();
         markUnencrypted(&obj1);
       }
       break;
@@ -1608,19 +1632,19 @@ void XRef::markUnencrypted(Object *obj) {
         dict = obj->getDict();
       }
       for (int i = 0; i < dict->getLength(); i++) {
-	obj1 = dict->getValNF(i);
+	obj1 = dict->getValNF(i).copy();
         markUnencrypted(&obj1);
       }
       break;
     }
     case objRef:
     {
-      Ref ref = obj->getRef();
+      const Ref ref = obj->getRef();
       XRefEntry *e = getEntry(ref.num);
       if (e->getFlag(XRefEntry::Unencrypted))
         return; // We've already been here: prevent infinite recursion
       e->setFlag(XRefEntry::Unencrypted, true);
-      obj1 = fetch(ref.num, ref.gen);
+      obj1 = fetch(ref);
       markUnencrypted(&obj1);
       break;
     }
@@ -1670,7 +1694,7 @@ void XRef::scanSpecialFlags() {
 
 void XRef::markUnencrypted() {
   // Mark objects referred from the Encrypt dict as Unencrypted
-  Object obj = trailerDict.dictLookupNF("Encrypt");
+  const Object &obj = trailerDict.dictLookupNF("Encrypt");
   if (obj.isRef()) {
     XRefEntry *e = getEntry(obj.getRefNum());
     e->setFlag(XRefEntry::Unencrypted, true);
